@@ -3,7 +3,9 @@
 #ifndef MESSAGE_H
 #define MESSAGE_H
 
-#include "definitions/definitions.h"
+// #include "definitions/definitions.h"
+
+#include "services/abstract/feedrtlbehavior.h"
 
 #include <QDataStream>
 #include <QDateTime>
@@ -26,17 +28,38 @@ struct RSSGUARD_DLLSPEC Enclosure {
 class RSSGUARD_DLLSPEC Enclosures {
   public:
     static QList<Enclosure> decodeEnclosuresFromString(const QString& enclosures_data);
+    static QJsonArray encodeEnclosuresToJson(const QList<Enclosure>& enclosures);
     static QString encodeEnclosuresToString(const QList<Enclosure>& enclosures);
 };
 
 class Feed;
 
+// Represents RSS, JSON or ATOM category (or tag, label - depending on terminology of each entry.
+class RSSGUARD_DLLSPEC MessageCategory : public QObject {
+    Q_OBJECT
+
+    Q_PROPERTY(QString title READ title)
+
+  public:
+    explicit MessageCategory(const QString& title);
+    MessageCategory(const MessageCategory& other);
+
+    QString title() const;
+
+    MessageCategory& operator=(const MessageCategory& other);
+
+  private:
+    QString m_title;
+};
+
 // Represents single message.
 class RSSGUARD_DLLSPEC Message {
   public:
     explicit Message();
 
-    void sanitize(const Feed* feed);
+    void sanitize(const Feed* feed, bool fix_future_datetimes);
+
+    QJsonObject toJson() const;
 
     // Creates Message from given record, which contains
     // row from query SELECT * FROM Messages WHERE ....;
@@ -54,6 +77,7 @@ class RSSGUARD_DLLSPEC Message {
     // to localtime when needed.
     QDateTime m_created;
     QString m_feedId;
+    QString m_feedTitle;
     int m_accountId;
     int m_id;
     QString m_customId;
@@ -62,14 +86,27 @@ class RSSGUARD_DLLSPEC Message {
     bool m_isImportant;
     bool m_isDeleted;
     double m_score;
+    RtlBehavior m_rtlBehavior;
     QList<Enclosure> m_enclosures;
 
+    // List of assigned labels.
+    // This field is only field when fetching entries of a feed.
+    QList<MessageCategory> m_categories;
+
     // List of custom IDs of labels assigned to this message.
     QList<Label*> m_assignedLabels;
 
+    QList<Label*> m_assignedLabelsByFilter;
+    QList<Label*> m_deassignedLabelsByFilter;
+
+    QStringList m_assignedLabelsIds;
+
     // Is true if "created" date was obtained directly
     // from the feed, otherwise is false
     bool m_createdFromFeed = false;
+
+    // Notice if the article was actually inserted/updated into DB when feed was fetched.
+    bool m_insertedUpdated = false;
 };
 
 inline bool operator==(const Message& lhs, const Message& rhs) {
