@@ -3,30 +3,33 @@
 #ifndef MESSAGEOBJECT_H
 #define MESSAGEOBJECT_H
 
-#include <QObject>
-
 #include "services/abstract/label.h"
 
-class MessageObject : public QObject {
-  Q_OBJECT
+#include <QObject>
 
-  Q_PROPERTY(QList<Label*> assignedLabels READ assignedLabels)
-  Q_PROPERTY(QList<Label*> availableLabels READ availableLabels)
-  Q_PROPERTY(QString feedCustomId READ feedCustomId)
-  Q_PROPERTY(int accountId READ accountId)
-  Q_PROPERTY(int id READ id)
-  Q_PROPERTY(QString customId READ customId)
-  Q_PROPERTY(QString title READ title WRITE setTitle)
-  Q_PROPERTY(QString url READ url WRITE setUrl)
-  Q_PROPERTY(QString author READ author WRITE setAuthor)
-  Q_PROPERTY(QString contents READ contents WRITE setContents)
-  Q_PROPERTY(QString rawContents READ rawContents WRITE setRawContents)
-  Q_PROPERTY(QDateTime created READ created WRITE setCreated)
-  Q_PROPERTY(double score READ score WRITE setScore)
-  Q_PROPERTY(bool isRead READ isRead WRITE setIsRead)
-  Q_PROPERTY(bool isImportant READ isImportant WRITE setIsImportant)
-  Q_PROPERTY(bool isDeleted READ isDeleted WRITE setIsDeleted)
-  Q_PROPERTY(bool runningFilterWhenFetching READ runningFilterWhenFetching)
+class MessageObject : public QObject {
+    Q_OBJECT
+
+    Q_PROPERTY(QList<MessageCategory> categories READ categories)
+    Q_PROPERTY(QList<Label*> assignedLabels READ assignedLabels)
+    Q_PROPERTY(QList<Label*> availableLabels READ availableLabels)
+    Q_PROPERTY(QString feedCustomId READ feedCustomId)
+    Q_PROPERTY(int accountId READ accountId)
+    Q_PROPERTY(int id READ id)
+    Q_PROPERTY(bool hasEnclosures READ hasEnclosures)
+    Q_PROPERTY(QString customId READ customId WRITE setCustomId)
+    Q_PROPERTY(QString title READ title WRITE setTitle)
+    Q_PROPERTY(QString url READ url WRITE setUrl)
+    Q_PROPERTY(QString author READ author WRITE setAuthor)
+    Q_PROPERTY(QString contents READ contents WRITE setContents)
+    Q_PROPERTY(QString rawContents READ rawContents WRITE setRawContents)
+    Q_PROPERTY(QDateTime created READ created WRITE setCreated)
+    Q_PROPERTY(bool createdIsMadeup READ createdIsMadeup WRITE setCreatedIsMadeup)
+    Q_PROPERTY(double score READ score WRITE setScore)
+    Q_PROPERTY(bool isRead READ isRead WRITE setIsRead)
+    Q_PROPERTY(bool isImportant READ isImportant WRITE setIsImportant)
+    Q_PROPERTY(bool isDeleted READ isDeleted WRITE setIsDeleted)
+    Q_PROPERTY(bool runningFilterWhenFetching READ runningFilterWhenFetching)
 
   public:
     enum class FilteringAction {
@@ -42,7 +45,7 @@ class MessageObject : public QObject {
 
     Q_ENUM(FilteringAction)
 
-    enum class DuplicationAttributeCheck {
+    enum class DuplicateCheck {
       // Message with same title in DB.
       SameTitle = 1,
 
@@ -64,22 +67,20 @@ class MessageObject : public QObject {
       SameCustomId = 32
     };
 
-    Q_ENUM(DuplicationAttributeCheck)
+    Q_ENUM(DuplicateCheck)
 
     explicit MessageObject(QSqlDatabase* db,
-                           const QString& feed_custom_id,
-                           int account_id,
-                           const QList<Label*>& available_labels,
+                           Feed* feed,
+                           ServiceRoot* account,
                            bool is_new_message,
                            QObject* parent = nullptr);
 
     void setMessage(Message* message);
 
     // Check if message is duplicate with another messages in DB.
-    // Parameter "attribute_check" is DuplicationAttributeCheck enum
-    // value casted to int.
-    Q_INVOKABLE bool isDuplicate(MessageObject::DuplicationAttributeCheck attribute_check) const;
-    Q_INVOKABLE bool isDuplicateWithAttribute(MessageObject::DuplicationAttributeCheck attribute_check) const;
+    Q_INVOKABLE bool isAlreadyInDatabase(MessageObject::DuplicateCheck attribute_check) const;
+    Q_INVOKABLE bool isDuplicate(MessageObject::DuplicateCheck attribute_check) const;
+    Q_INVOKABLE bool isDuplicateWithAttribute(MessageObject::DuplicateCheck attribute_check) const;
 
     // Adds given label to list of assigned labels to this message.
     // Returns true if label was assigned now or if the message already has it assigned.
@@ -89,11 +90,24 @@ class MessageObject : public QObject {
     // Returns true if label was now removed or if it is not assigned to the message at all.
     Q_INVOKABLE bool deassignLabel(const QString& label_custom_id) const;
 
+    // Returns label custom ID given label title.
+    Q_INVOKABLE QString findLabelId(const QString& label_title) const;
+
+    // Returns label custom ID given label title or creates
+    // the label if it does not exist.
+    Q_INVOKABLE QString createLabelId(const QString& title, const QString& hex_color = {});
+
+    // Add multimedia attachment to the message.
+    Q_INVOKABLE void addEnclosure(const QString& url, const QString& mime_type) const;
+
     // Returns list of assigned and available messages.
     QList<Label*> assignedLabels() const;
     QList<Label*> availableLabels() const;
 
+    QList<MessageCategory> categories() const;
+
     bool runningFilterWhenFetching() const;
+    bool hasEnclosures() const;
 
     // Generic Message's properties bindings.
     QString feedCustomId() const;
@@ -101,6 +115,7 @@ class MessageObject : public QObject {
     int accountId() const;
 
     QString customId() const;
+    void setCustomId(const QString& custom_id);
 
     int id() const;
 
@@ -122,6 +137,9 @@ class MessageObject : public QObject {
     QDateTime created() const;
     void setCreated(const QDateTime& created);
 
+    bool createdIsMadeup() const;
+    void setCreatedIsMadeup(bool madeup);
+
     bool isRead() const;
     void setIsRead(bool is_read);
 
@@ -136,6 +154,10 @@ class MessageObject : public QObject {
 
   private:
     QSqlDatabase* m_db;
+
+    Feed* m_feed;
+    ServiceRoot* m_account;
+
     QString m_feedCustomId;
     int m_accountId;
     Message* m_message;
@@ -143,14 +165,4 @@ class MessageObject : public QObject {
     bool m_runningAfterFetching;
 };
 
-inline MessageObject::DuplicationAttributeCheck operator|(MessageObject::DuplicationAttributeCheck lhs,
-                                                          MessageObject::DuplicationAttributeCheck rhs) {
-  return static_cast<MessageObject::DuplicationAttributeCheck>(int(lhs) | int(rhs));
-}
-
-inline MessageObject::DuplicationAttributeCheck operator&(MessageObject::DuplicationAttributeCheck lhs,
-                                                          MessageObject::DuplicationAttributeCheck rhs) {
-  return static_cast<MessageObject::DuplicationAttributeCheck>(int(lhs) & int(rhs));
-}
-
 #endif // MESSAGEOBJECT_H
