@@ -3,23 +3,26 @@
 #ifndef FORMSETTINGS_H
 #define FORMSETTINGS_H
 
-#include <QDialog>
-
 #include "ui_formsettings.h"
 
+#include <QDialog>
+
 class Settings;
 class SettingsPanel;
 
 class FormSettings : public QDialog {
-  Q_OBJECT
+    Q_OBJECT
 
   public:
-
     // Constructors and destructors.
     explicit FormSettings(QWidget& parent);
     virtual ~FormSettings();
 
+  public slots:
+    void reject();
+
   private slots:
+    void openSettingsCategory(int category);
 
     // Saves settings into global configuration.
     void saveSettings();
@@ -31,7 +34,6 @@ class FormSettings : public QDialog {
 
     Ui::FormSettings m_ui;
     QPushButton* m_btnApply;
-
     QList<SettingsPanel*> m_panels;
     Settings& m_settings;
 };
