@@ -3,12 +3,11 @@
 #ifndef TABWIDGET_H
 #define TABWIDGET_H
 
-#include <QTabWidget>
-
 #include "core/message.h"
 #include "gui/tabbar.h"
 #include "gui/tabcontent.h"
 
+#include <QTabWidget>
 #include <QUrl>
 
 class QMenu;
@@ -17,23 +16,25 @@ class RootItem;
 class FeedMessageViewer;
 
 class TabWidget : public QTabWidget {
-  Q_OBJECT
+    Q_OBJECT
 
   public:
-
     // Constructors and destructors.
     explicit TabWidget(QWidget* parent = nullptr);
     virtual ~TabWidget();
 
     // Manimulators for tabs.
-    int addTab(TabContent* widget, const QString&,
+    int addTab(TabContent* widget, const QString&, TabBar::TabType type = TabBar::TabType::NonClosable);
+    int addTab(TabContent* widget,
+               const QIcon& icon,
+               const QString& label,
                TabBar::TabType type = TabBar::TabType::NonClosable);
-    int addTab(TabContent* widget, const QIcon& icon,
-               const QString& label, TabBar::TabType type = TabBar::TabType::NonClosable);
-    int insertTab(int index, QWidget* widget, const QString& label,
-                  TabBar::TabType type = TabBar::TabType::Closable);
-    int insertTab(int index, QWidget* widget, const QIcon& icon,
-                  const QString& label, TabBar::TabType type = TabBar::TabType::NonClosable);
+    int insertTab(int index, QWidget* widget, const QString& label, TabBar::TabType type = TabBar::TabType::Closable);
+    int insertTab(int index,
+                  QWidget* widget,
+                  const QIcon& icon,
+                  const QString& label,
+                  TabBar::TabType type = TabBar::TabType::NonClosable);
     void removeTab(int index, bool clear_from_memory);
 
     // Returns tab bar.
@@ -55,23 +56,32 @@ class TabWidget : public QTabWidget {
     FeedMessageViewer* feedMessageViewer() const;
 
   public slots:
+    void scrollUpCurrentBrowser();
+    void scrollDownCurrentBrowser();
 
     // Called when number of tab pages changes.
-    void checkTabBarVisibility();
+    void checkCornerButtonVisibility();
+    void updateAppearance();
 
     // Tab closing.
     bool closeTab(int index);
+    void closeTabWithSender();
     void closeAllTabsExceptCurrent();
     void closeAllTabs();
+    void closeCurrentTab();
 
     // Displays download manager.
     void showDownloadManager();
 
-    int addNewspaperView(RootItem* root, const QList<Message>& messages);
+    int addSingleMessageView(RootItem* root, const Message& message);
 
     // Adds new WebBrowser tab to global TabWidget.
     int addEmptyBrowser();
 
+#if defined(ENABLE_MEDIAPLAYER)
+    int addMediaPlayer(const QString& url, bool make_active);
+#endif
+
     // Adds new WebBrowser with link. This is used when user
     // selects to "Open link in new tab.".
     int addLinkedBrowser(const QUrl& initial_url = QUrl());
@@ -79,6 +89,7 @@ class TabWidget : public QTabWidget {
 
     // General method for adding WebBrowsers.
     int addBrowser(bool move_after_current, bool make_active, const QUrl& initial_url = QUrl());
+    int addBrowser(bool move_after_current, bool make_active, WebBrowser* browser);
 
     void gotoNextTab();
     void gotoPreviousTab();
