@@ -2,11 +2,13 @@
 
 #include "miscellaneous/systemfactory.h"
 
+#include "3rd-party/boolinq/boolinq.h"
+#include "exceptions/applicationexception.h"
 #include "gui/dialogs/formmain.h"
 #include "gui/dialogs/formupdate.h"
 #include "miscellaneous/application.h"
+#include "miscellaneous/settings.h"
 #include "miscellaneous/systemfactory.h"
-#include "network-web/networkfactory.h"
 
 #if defined(Q_OS_WIN)
 #include <QSettings>
@@ -16,13 +18,12 @@
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
-#include <QFuture>
-#include <QFutureWatcher>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QProcess>
 #include <QString>
+#include <QVersionNumber>
 
 using UpdateCheck = QPair<UpdateInfo, QNetworkReply::NetworkError>;
 
@@ -47,10 +48,7 @@ SystemFactory::AutoStartStatus SystemFactory::autoStartStatus() const {
 #if defined(Q_OS_WIN)
   QSettings registry_key(QSL("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"),
                          QSettings::Format::NativeFormat);
-  const bool autostart_enabled = registry_key.value(QSL(APP_LOW_NAME),
-                                                    QString()).toString().replace(QL1C('\\'),
-                                                                                  QL1C('/')) ==
-                                 Application::applicationFilePath();
+  const bool autostart_enabled = registry_key.contains(QSL(APP_LOW_NAME));
 
   if (autostart_enabled) {
     return AutoStartStatus::Enabled;
@@ -58,8 +56,8 @@ SystemFactory::AutoStartStatus SystemFactory::autoStartStatus() const {
   else {
     return AutoStartStatus::Disabled;
   }
-#elif defined(Q_OS_LINUX)
-  // Use proper freedesktop.org way to auto-start the application on Linux.
+#elif defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+  // Use proper freedesktop.org way to auto-start the application.
   // INFO: http://standards.freedesktop.org/autostart-spec/latest/
   const QString desktop_file_location = autostartDesktopFileLocation();
 
@@ -86,7 +84,7 @@ SystemFactory::AutoStartStatus SystemFactory::autoStartStatus() const {
 #endif
 }
 
-#if defined(Q_OS_LINUX)
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
 QString SystemFactory::autostartDesktopFileLocation() const {
   const QString xdg_config_path(qgetenv("XDG_CONFIG_HOME"));
   QString desktop_file_location;
@@ -94,7 +92,7 @@ QString SystemFactory::autostartDesktopFileLocation() const {
   if (!xdg_config_path.isEmpty()) {
     // XDG_CONFIG_HOME variable is specified. Look for .desktop file
     // in 'autostart' subdirectory.
-    desktop_file_location = xdg_config_path + QSL("/autostart/") + APP_DESKTOP_ENTRY_FILE;
+    desktop_file_location = xdg_config_path + QSL("/autostart/") + APP_REVERSE_NAME + QSL(".desktop");
   }
   else {
     // Desired variable is not set, look for the default 'autostart' subdirectory.
@@ -103,7 +101,7 @@ QString SystemFactory::autostartDesktopFileLocation() const {
     if (!home_directory.isEmpty()) {
       // Home directory exists. Check if target .desktop file exists and
       // return according status.
-      desktop_file_location = home_directory + QSL("/.config/autostart/") + APP_DESKTOP_ENTRY_FILE;
+      desktop_file_location = home_directory + QSL("/.config/autostart/") + APP_REVERSE_NAME + QSL(".desktop");
     }
   }
 
@@ -121,13 +119,28 @@ bool SystemFactory::setAutoStartStatus(AutoStartStatus new_status) {
   }
 
 #if defined(Q_OS_WIN)
-  QSettings registry_key(QSL("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"), QSettings::NativeFormat);
+  QSettings registry_key(QSL("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"),
+                         QSettings::Format::NativeFormat);
 
   switch (new_status) {
-    case AutoStartStatus::Enabled:
-      registry_key.setValue(QSL(APP_LOW_NAME),
-                            Application::applicationFilePath().replace(QL1C('/'), QL1C('\\')));
+    case AutoStartStatus::Enabled: {
+      QStringList args = qApp->rawCliArgs();
+      auto std_args = boolinq::from(args.begin(), args.end())
+                        .select([](const QString& arg) {
+                          if (arg.contains(QL1S(" ")) && !arg.startsWith(QL1S("\""))) {
+                            return QSL("\"%1\"").arg(arg);
+                          }
+                          else {
+                            return arg;
+                          }
+                        })
+                        .toStdList();
+      args = FROM_STD_LIST(QStringList, std_args);
+
+      QString app_run_line = args.join(QL1C(' '));
+      registry_key.setValue(QSL(APP_LOW_NAME), app_run_line);
       return true;
+    }
 
     case AutoStartStatus::Disabled:
       registry_key.remove(QSL(APP_LOW_NAME));
@@ -136,7 +149,7 @@ bool SystemFactory::setAutoStartStatus(AutoStartStatus new_status) {
     default:
       return false;
   }
-#elif defined(Q_OS_LINUX)
+#elif defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
   // Note that we expect here that no other program uses
   // "rssguard.desktop" desktop file.
   const QString destination_file = autostartDesktopFileLocation();
@@ -154,9 +167,45 @@ bool SystemFactory::setAutoStartStatus(AutoStartStatus new_status) {
         return false;
       }
 
-      const QString source_autostart_desktop_file = QString(APP_DESKTOP_ENTRY_PATH) + QDir::separator() + APP_DESKTOP_SOURCE_ENTRY_FILE;
+      const QString source_autostart_desktop_file =
+        QString(APP_DESKTOP_ENTRY_PATH) + QDir::separator() + APP_DESKTOP_ENTRY_FILE;
 
-      return QFile::copy(source_autostart_desktop_file, destination_file);
+      try {
+        QString desktop_file_contents = QString::fromUtf8(IOFactory::readFile(source_autostart_desktop_file));
+
+        QStringList args = qApp->rawCliArgs();
+        auto std_args = boolinq::from(args.begin(), args.end())
+                          .select([](const QString& arg) {
+                            if (arg.contains(QL1S(" ")) && !arg.startsWith(QL1S("\""))) {
+                              return QSL("\"%1\"").arg(arg);
+                            }
+                            else {
+                              return arg;
+                            }
+                          })
+                          .toStdList();
+        args = FROM_STD_LIST(QStringList, std_args);
+
+#if defined(IS_FLATPAK_BUILD)
+        const QString flatpak_run = QSL("flatpak run %1").arg(APP_REVERSE_NAME);
+
+        args = args.mid(1);
+        args.prepend(flatpak_run);
+#endif
+
+        desktop_file_contents.replace("Exec=@APP_LOW_NAME@", QSL("Exec=") + args.join(QL1C(' ')));
+        desktop_file_contents.replace("@APPDATA_SUMMARY@", APPDATA_SUMMARY);
+        desktop_file_contents.replace("@APPDATA_NAME@", APPDATA_NAME);
+        desktop_file_contents.replace("@APP_REVERSE_NAME@", APP_REVERSE_NAME);
+        desktop_file_contents.replace("@APP_LOW_NAME@", APP_LOW_NAME);
+
+        IOFactory::writeFile(destination_file, desktop_file_contents.toUtf8());
+      }
+      catch (const ApplicationException& ex) {
+        return false;
+      }
+
+      return true;
     }
 
     case AutoStartStatus::Disabled:
@@ -171,10 +220,10 @@ bool SystemFactory::setAutoStartStatus(AutoStartStatus new_status) {
 }
 
 QString SystemFactory::loggedInUser() const {
-  QString name = qgetenv("USER");
+  QString name = qEnvironmentVariable("USER");
 
   if (name.isEmpty()) {
-    name = qgetenv("USERNAME");
+    name = qEnvironmentVariable("USERNAME");
   }
 
   if (name.isEmpty()) {
@@ -203,58 +252,36 @@ void SystemFactory::checkForUpdates() const {
 }
 
 void SystemFactory::checkForUpdatesOnStartup() {
+#if !defined(NO_UPDATE_CHECK)
   if (qApp->settings()->value(GROUP(General), SETTING(General::UpdateOnStartup)).toBool()) {
-    QObject::connect(qApp->system(), &SystemFactory::updatesChecked,
-                     this, [&](const QPair<QList<UpdateInfo>, QNetworkReply::NetworkError>& updates) {
-      QObject::disconnect(qApp->system(), &SystemFactory::updatesChecked, this, nullptr);
+    QObject::connect(qApp->system(),
+                     &SystemFactory::updatesChecked,
+                     this,
+                     [&](const QPair<QList<UpdateInfo>, QNetworkReply::NetworkError>& updates) {
+                       QObject::disconnect(qApp->system(), &SystemFactory::updatesChecked, this, nullptr);
 
-      if (!updates.first.isEmpty() &&
-          updates.second == QNetworkReply::NetworkError::NoError &&
-          SystemFactory::isVersionNewer(updates.first.at(0).m_availableVersion, QSL(APP_VERSION))) {
-        qApp->showGuiMessage(Notification::Event::NewAppVersionAvailable,
-                             QObject::tr("New version available"),
-                             QObject::tr("Click the bubble for more information."),
-                             QSystemTrayIcon::Information, {}, {},
-                             tr("See new version info"),
-                             [] {
-          FormUpdate(qApp->mainForm()).exec();
-        });
-      }
-    });
-    qApp->system()->checkForUpdates();
+                       if (!updates.first.isEmpty() && updates.second == QNetworkReply::NetworkError::NoError &&
+                           SystemFactory::isVersionNewer(updates.first.at(0).m_availableVersion, QSL(APP_VERSION))) {
+                         qApp->showGuiMessage(Notification::Event::NewAppVersionAvailable,
+                                              {QObject::tr("New version available"),
+                                               QObject::tr("Click the bubble for more information."),
+                                               QSystemTrayIcon::Information},
+                                              {},
+                                              {tr("See new version info"), [] {
+                                                 FormUpdate(qApp->mainForm()).exec();
+                                               }});
+                       }
+                     });
+    checkForUpdates();
   }
+#endif
 }
 
 bool SystemFactory::isVersionNewer(const QString& new_version, const QString& base_version) {
-  QStringList base_version_tkn = base_version.split(QL1C('.'));
-  QStringList new_version_tkn = new_version.split(QL1C('.'));
+  QVersionNumber nw = QVersionNumber::fromString(new_version);
+  QVersionNumber bs = QVersionNumber::fromString(base_version);
 
-  while (!base_version_tkn.isEmpty() && !new_version_tkn.isEmpty()) {
-    const int base_number = base_version_tkn.takeFirst().toInt();
-    const int new_number = new_version_tkn.takeFirst().toInt();
-
-    if (new_number > base_number) {
-      // New version is indeed higher that current version.
-      return true;
-    }
-    else if (new_number < base_number) {
-      return false;
-    }
-  }
-
-  // Versions are either the same or they have unequal sizes.
-  if (base_version_tkn.isEmpty() && new_version_tkn.isEmpty()) {
-    // Versions are the same.
-    return false;
-  }
-  else {
-    if (new_version_tkn.isEmpty()) {
-      return false;
-    }
-    else {
-      return new_version_tkn.join(QString()).toInt() > 0;
-    }
-  }
+  return nw > bs;
 }
 
 bool SystemFactory::isVersionEqualOrNewer(const QString& new_version, const QString& base_version) {
@@ -263,8 +290,7 @@ bool SystemFactory::isVersionEqualOrNewer(const QString& new_version, const QStr
 
 bool SystemFactory::openFolderFile(const QString& file_path) {
 #if defined(Q_OS_WIN)
-  return QProcess::startDetached(QSL("explorer.exe"),
-                                 { "/select,", QDir::toNativeSeparators(file_path) });
+  return QProcess::startDetached(QSL("explorer.exe"), {"/select,", QDir::toNativeSeparators(file_path)});
 #else
   const QString folder = QDir::toNativeSeparators(QFileInfo(file_path).absoluteDir().absolutePath());
 
