@@ -5,8 +5,10 @@
 
 #include <QVariantList>
 
-template <class T> static QVariant toVariant(const QList<T>& list) {
-  QVariantList variant_list; variant_list.reserve(list.size());
+template <class T>
+static QVariant toVariant(const QList<T>& list) {
+  QVariantList variant_list;
+  variant_list.reserve(list.size());
 
   for (const auto& v : list) {
     variant_list.append(v);
@@ -15,10 +17,11 @@ template <class T> static QVariant toVariant(const QList<T>& list) {
   return variant_list;
 }
 
-template <class T> static QList<T> toList(const QVariant& qv) {
-  QList <T> data_list;
+template <class T>
+static QList<T> toList(const QVariant& qv) {
+  QList<T> data_list;
 
-  foreach(QVariant v, qv.value<QVariantList>()) {
+  foreach (QVariant v, qv.value<QVariantList>()) {
     data_list << v.value<T>();
   }
 
