@@ -4,9 +4,11 @@
 
 #include "definitions/definitions.h"
 
+#if defined(NO_LITE)
 AdblockRequestInfo::AdblockRequestInfo(const QWebEngineUrlRequestInfo& webengine_info) {
   initialize(webengine_info);
 }
+#endif
 
 AdblockRequestInfo::AdblockRequestInfo(const QUrl& url) {
   initialize(url);
@@ -44,6 +46,7 @@ void AdblockRequestInfo::setRequestMethod(const QByteArray& request_method) {
   m_requestMethod = request_method;
 }
 
+#if defined(NO_LITE)
 void AdblockRequestInfo::initialize(const QWebEngineUrlRequestInfo& webengine_info) {
   setFirstPartyUrl(webengine_info.firstPartyUrl());
   setRequestMethod(webengine_info.requestMethod());
@@ -51,13 +54,6 @@ void AdblockRequestInfo::initialize(const QWebEngineUrlRequestInfo& webengine_in
   setResourceType(convertResourceType(webengine_info.resourceType()));
 }
 
-void AdblockRequestInfo::initialize(const QUrl& url) {
-  setFirstPartyUrl(url);
-  setRequestMethod(QSL("GET").toLocal8Bit());
-  setRequestUrl(url);
-  setResourceType(convertResourceType(QWebEngineUrlRequestInfo::ResourceType::ResourceTypeMainFrame));
-}
-
 QString AdblockRequestInfo::convertResourceType(QWebEngineUrlRequestInfo::ResourceType rt) const {
   switch (rt) {
     case QWebEngineUrlRequestInfo::ResourceType::ResourceTypeStylesheet:
@@ -95,3 +91,17 @@ QString AdblockRequestInfo::convertResourceType(QWebEngineUrlRequestInfo::Resour
       return {};
   }
 }
+
+#endif
+
+void AdblockRequestInfo::initialize(const QUrl& url) {
+  setFirstPartyUrl(url);
+  setRequestMethod(QSL("GET").toLocal8Bit());
+  setRequestUrl(url);
+
+#if defined(NO_LITE)
+  setResourceType(convertResourceType(QWebEngineUrlRequestInfo::ResourceType::ResourceTypeMainFrame));
+#else
+  setResourceType(QSL("main_frame"));
+#endif
+}
