@@ -2,17 +2,16 @@
 
 #include "services/abstract/label.h"
 
-#include "gui/dialogs/formaddeditlabel.h"
-#include "miscellaneous/application.h"
 #include "database/databasefactory.h"
 #include "database/databasequeries.h"
+#include "definitions/globals.h"
+#include "miscellaneous/application.h"
 #include "services/abstract/cacheforserviceroot.h"
-#include "services/abstract/labelsnode.h"
+#include "services/abstract/gui/formaddeditlabel.h"
 #include "services/abstract/serviceroot.h"
 
 #include <QPainter>
 #include <QPainterPath>
-#include <QThread>
 
 Label::Label(const QString& name, const QColor& color, RootItem* parent_item) : Label(parent_item) {
   setColor(color);
@@ -28,7 +27,7 @@ QColor Label::color() const {
 }
 
 void Label::setColor(const QColor& color) {
-  setIcon(generateIcon(color));
+  setIcon(IconFactory::generateIcon(color));
   m_color = color;
 }
 
@@ -41,29 +40,14 @@ int Label::countOfAllMessages() const {
 }
 
 bool Label::canBeEdited() const {
-  return (getParentServiceRoot()->supportedLabelOperations() & ServiceRoot::LabelOperation::Editing) ==
-         ServiceRoot::LabelOperation::Editing;
-}
-
-bool Label::editViaGui() {
-  FormAddEditLabel form(qApp->mainFormWidget());
-
-  if (form.execForEdit(this)) {
-    QSqlDatabase db = qApp->database()->driver()->connection(metaObject()->className());
-
-    return DatabaseQueries::updateLabel(db, this);
-  }
-  else {
-    return false;
-  }
+  return Globals::hasFlag(getParentServiceRoot()->supportedLabelOperations(), ServiceRoot::LabelOperation::Editing);
 }
 
 bool Label::canBeDeleted() const {
-  return (getParentServiceRoot()->supportedLabelOperations() & ServiceRoot::LabelOperation::Deleting) ==
-         ServiceRoot::LabelOperation::Deleting;
+  return Globals::hasFlag(getParentServiceRoot()->supportedLabelOperations(), ServiceRoot::LabelOperation::Deleting);
 }
 
-bool Label::deleteViaGui() {
+bool Label::deleteItem() {
   QSqlDatabase db = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::deleteLabel(db, this)) {
@@ -76,17 +60,17 @@ bool Label::deleteViaGui() {
 }
 
 void Label::updateCounts(bool including_total_count) {
-  bool is_main_thread = QThread::currentThread() == qApp->thread();
-  QSqlDatabase database = is_main_thread ?
-                          qApp->database()->driver()->connection(metaObject()->className()) :
-                          qApp->database()->driver()->connection(QSL("feed_upd"));
+  QSqlDatabase database = qApp->database()->driver()->threadSafeConnection(metaObject()->className());
   int account_id = getParentServiceRoot()->accountId();
 
+  // TODO: slow
+  auto ac = DatabaseQueries::getMessageCountsForLabel(database, this, account_id);
+
   if (including_total_count) {
-    setCountOfAllMessages(DatabaseQueries::getMessageCountsForLabel(database, this, account_id, true));
+    setCountOfAllMessages(ac.m_total);
   }
 
-  setCountOfUnreadMessages(DatabaseQueries::getMessageCountsForLabel(database, this, account_id, false));
+  setCountOfUnreadMessages(ac.m_unread);
 }
 
 QList<Message> Label::undeletedMessages() const {
@@ -95,43 +79,27 @@ QList<Message> Label::undeletedMessages() const {
   return DatabaseQueries::getUndeletedMessagesWithLabel(database, this);
 }
 
-QIcon Label::generateIcon(const QColor& color) {
-  QPixmap pxm(64, 64);
+void Label::assignToMessage(const Message& msg, bool reload_model) {
+  QSqlDatabase database = qApp->database()->driver()->threadSafeConnection(metaObject()->className());
 
-  pxm.fill(Qt::GlobalColor::transparent);
-
-  QPainter paint(&pxm);
-
-  paint.setBrush(color);
-  paint.setPen(Qt::GlobalColor::transparent);
-  paint.drawEllipse(pxm.rect().marginsRemoved(QMargins(2, 2, 2, 2)));
-
-  return pxm;
-}
-
-void Label::assignToMessage(const Message& msg) {
-  bool is_main_thread = QThread::currentThread() == qApp->thread();
-  QSqlDatabase database = is_main_thread ?
-                          qApp->database()->driver()->connection(metaObject()->className()) :
-                          qApp->database()->driver()->connection(QSL("feed_upd"));
-
-  if (getParentServiceRoot()->onBeforeLabelMessageAssignmentChanged({ this }, { msg }, true)) {
+  if (getParentServiceRoot()->onBeforeLabelMessageAssignmentChanged({this}, {msg}, true)) {
     DatabaseQueries::assignLabelToMessage(database, this, msg);
 
-    getParentServiceRoot()->onAfterLabelMessageAssignmentChanged({ this }, { msg }, true);
+    if (reload_model) {
+      getParentServiceRoot()->onAfterLabelMessageAssignmentChanged({this}, {msg}, true);
+    }
   }
 }
 
-void Label::deassignFromMessage(const Message& msg) {
-  bool is_main_thread = QThread::currentThread() == qApp->thread();
-  QSqlDatabase database = is_main_thread ?
-                          qApp->database()->driver()->connection(metaObject()->className()) :
-                          qApp->database()->driver()->connection(QSL("feed_upd"));
+void Label::deassignFromMessage(const Message& msg, bool reload_model) {
+  QSqlDatabase database = qApp->database()->driver()->threadSafeConnection(metaObject()->className());
 
-  if (getParentServiceRoot()->onBeforeLabelMessageAssignmentChanged({ this }, { msg }, false)) {
+  if (getParentServiceRoot()->onBeforeLabelMessageAssignmentChanged({this}, {msg}, false)) {
     DatabaseQueries::deassignLabelFromMessage(database, this, msg);
 
-    getParentServiceRoot()->onAfterLabelMessageAssignmentChanged({ this }, { msg }, false);
+    if (reload_model) {
+      getParentServiceRoot()->onAfterLabelMessageAssignmentChanged({this}, {msg}, false);
+    }
   }
 }
 
@@ -149,7 +117,7 @@ bool Label::cleanMessages(bool clear_only_read) {
 
   if (DatabaseQueries::cleanLabelledMessages(database, clear_only_read, this)) {
     service->updateCounts(true);
-    service->itemChanged(service->getSubTree());
+    service->itemChanged(service->getSubTree<RootItem>());
     service->requestReloadMessageList(true);
     return true;
   }
@@ -163,14 +131,14 @@ bool Label::markAsReadUnread(RootItem::ReadStatus status) {
   auto* cache = dynamic_cast<CacheForServiceRoot*>(service);
 
   if (cache != nullptr) {
-    cache->addMessageStatesToCache(service->customIDSOfMessagesForItem(this), status);
+    cache->addMessageStatesToCache(service->customIDSOfMessagesForItem(this, status), status);
   }
 
   QSqlDatabase database = qApp->database()->driver()->connection(metaObject()->className());
 
   if (DatabaseQueries::markLabelledMessagesReadUnread(database, this, status)) {
     service->updateCounts(false);
-    service->itemChanged(service->getSubTree());
+    service->itemChanged(service->getSubTree<RootItem>());
     service->requestReloadMessageList(status == RootItem::ReadStatus::Read);
     return true;
   }
